import { Button, Link as MuiLink, Tooltip, Typography } from "@mui/material";
import React, { useState, useEffect, useContext, useCallback } from "react";
import { IOCDetails } from "../IOCDetails";
import { DeployIOC } from "../DeployIOC";
import { UndeployIOC } from "../UndeployIOC";
import { SimpleModal } from "../../common/SimpleModal/SimpleModal";
import {
  useUpdateAndDeployIoc,
  useCreateUndeployment,
  useJobById
} from "../../../api/SwaggerApi";
import { userContext, SimpleAccordion } from "@ess-ics/ce-ui-common";
import AlertMessages from "../AlertMessages";
import AccessControl from "../../auth/AccessControl";
import { DeploymentStatus } from "../../../api/DataTypes";
import { IOCService } from "../IOCService";
import useUrlState from "@ahooksjs/use-url-state";
import { serialize, deserialize } from "../../common/URLState/URLState";
import { JobTable } from "../../Job";

export function IOCManage({
  ioc,
  getIOC,
  buttonDisabled,
  currentCommand,
  operations,
  operationsLoading,
  getOperations,
  setButtonDisabled,
  pagination,
  onPage
}) {
  const { user } = useContext(userContext);
  const [state, setState] = useUrlState(
    {
      log_open: "true"
    },
    { navigateMode: "replace" }
  );

  const [deployDialogOpen, setDeployDialogOpen] = useState(false);
  const [undeployDialogOpen, setUndeployDialogOpen] = useState(false);
  const [deploymentJob, getDeploymentJobById] = useJobById();

  useEffect(() => {
    if (ioc.activeDeployment?.awxJobId) {
      getDeploymentJobById(ioc.activeDeployment?.awxJobId);
    }
  }, [getDeploymentJobById, ioc.activeDeployment?.awxJobId]);

  const closeDeployModal = () => {
    setDeployDialogOpen(false);
    getIOC();
    // history.push(window.location.pathname);
  };

  const closeUndeployModal = () => {
    setUndeployDialogOpen(false);
  };

  const getSubset = useCallback(
    (ioc) => {
      ioc = { ...ioc };
      const { sourceUrl: git } = ioc;

      // Show START/STOP components only when IOC was deployed SUCCESSFULLY
      const deploymentStatus = new DeploymentStatus(
        ioc.activeDeployment,
        deploymentJob
      );
      const showControls = deploymentStatus.wasSuccessful();

      let subset = {
        "IOC name": (
          <Typography>
            <MuiLink
              href={`${window.NAMING_ADDRESS}/devices.xhtml?i=2&deviceName=${ioc.namingName}`}
              target="_blank"
              rel="noreferrer"
              underline="hover"
            >
              {ioc.namingName}
            </MuiLink>
          </Typography>
        ),
        git: (
          <Typography>
            <MuiLink
              href={git}
              target="_blank"
              rel="noreferrer"
              underline="hover"
            >
              {git}
            </MuiLink>
          </Typography>
        )
      };

      if (user) {
        subset["IOC Service Controls"] = showControls ? (
          <IOCService
            {...{
              ioc,
              currentCommand,
              getOperations,
              buttonDisabled,
              setButtonDisabled,
              jobLazyParams: pagination
            }}
          />
        ) : (
          "Can't be controlled, because IOC is not (yet) deployed succesfully"
        );
      }

      return subset;
    },
    [
      buttonDisabled,
      pagination,
      currentCommand,
      deploymentJob,
      getOperations,
      setButtonDisabled,
      user
    ]
  );

  let content = <></>;
  if (ioc) {
    const managedIOC = { ...ioc };

    const formInit = {
      name: ioc.namingName,
      description: ioc.description,
      git: ioc.sourceUrl,
      gitProjectId: ioc.gitProjectId
    };

    if (ioc.activeDeployment) {
      formInit.version = ioc.activeDeployment.sourceVersion;
      formInit.shortVersion = ioc.activeDeployment.sourceVersionShort;
      formInit.csEntryHost = {
        fqdn: ioc.activeDeployment.host.fqdn,
        id: ioc.activeDeployment.host.csEntryId
      };
    }

    let disabledDebployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledDebployButtonTitle =
        "There is an ongoing operation, you can not deploy the IOC right now";
    }

    let disabledUndeployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledUndeployButtonTitle =
        "There is an ongoing operation, you can not undeploy the IOC right now";
    }

    content = (
      <>
        <IOCDetails
          ioc={managedIOC}
          getSubset={getSubset}
          alert={<AlertMessages alerts={managedIOC.alerts} />}
          buttons={
            <>
              <Tooltip title={disabledDebployButtonTitle}>
                <span>
                  <Button
                    variant="contained"
                    color="secondary"
                    onClick={() => {
                      setDeployDialogOpen(true);
                    }}
                    disabled={buttonDisabled || ioc.operationInProgress}
                  >
                    {ioc.activeDeployment ? "Deploy revision" : "Deploy"}
                  </Button>
                </span>
              </Tooltip>
              {ioc.activeDeployment ? (
                <Tooltip title={disabledUndeployButtonTitle}>
                  <span>
                    <Button
                      variant="contained"
                      color="secondary"
                      style={{ marginRight: 20 }}
                      onClick={() => {
                        setUndeployDialogOpen(true);
                      }}
                      disabled={buttonDisabled || ioc.operationInProgress}
                    >
                      Undeploy
                    </Button>
                  </span>
                </Tooltip>
              ) : (
                <></>
              )}
            </>
          }
        />
        <AccessControl
          allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}
        >
          <SimpleAccordion
            summary="Log"
            expanded={deserialize(state.log_open)}
            onChange={(event, expanded) =>
              setState({ log_open: serialize(expanded) })
            }
          >
            <JobTable
              jobs={operations}
              pagination={pagination}
              onPage={onPage}
              loading={operationsLoading}
            />
          </SimpleAccordion>
          <SimpleModal
            open={deployDialogOpen}
            setOpen={setDeployDialogOpen}
          >
            <DeployIOC
              open={deployDialogOpen}
              setOpen={setDeployDialogOpen}
              submitCallback={closeDeployModal}
              init={formInit}
              hook={useUpdateAndDeployIoc.bind(null, ioc.id)}
              hasActiveDeployment={ioc.activeDeployment}
            />
          </SimpleModal>
          <SimpleModal
            open={undeployDialogOpen}
            setOpen={setUndeployDialogOpen}
          >
            <UndeployIOC
              open={undeployDialogOpen}
              setOpen={setUndeployDialogOpen}
              submitCallback={closeUndeployModal}
              ioc={ioc}
              hook={useCreateUndeployment.bind(null, ioc.id)}
            />
          </SimpleModal>
        </AccessControl>
      </>
    );
  }

  return <>{content}</>;
}
