import React, { useState, useEffect, useCallback } from "react";
import { styled } from "@mui/material/styles";
import { useLokiSysLog, useLokiProcServLog } from "../../../api/SwaggerApi";
import {
  Grid,
  Select,
  MenuItem,
  InputLabel,
  FormControl,
  LinearProgress,
  Container
} from "@mui/material";
import { formatDate } from "../Helper.js";
import { Console } from "../Console/Console";
import { useSafePolling } from "../../../hooks/Polling";
import { useCustomSnackbar } from "../snackbar/Snackbar";
import { closeSnackbar } from "notistack";

const PREFIX = "LokiPanel";

const classes = {
  formControl: `${PREFIX}-formControl`,
  selectEmpty: `${PREFIX}-selectEmpty`,
  deployed: `${PREFIX}-deployed`,
  undeployed: `${PREFIX}-undeployed`
};

const Root = styled("html")(({ theme }) => ({
  [`& .${classes.formControl}`]: {
    margin: theme.spacing(0),
    minWidth: 120
  },

  [`& .${classes.selectEmpty}`]: {
    marginTop: theme.spacing(2)
  },

  [`& .${classes.deployed}`]: {
    backgroundColor: "black",
    msUserSelect: "text",
    MozUserSelect: "text",
    WebkitUserSelect: "text"
  },

  [`& .${classes.undeployed}`]: {
    backgroundColor: "gray",
    msUserSelect: "none",
    MozUserSelect: "none",
    WebkitUserSelect: "none"
  }
}));

/**
 * host: the host URL which will be the qery parameter for backend call
 * iocName: name of ioc (optional). Used when querying procServ log
 * isSyslog: boolean, if true -> SysLog will be queried, if false -> procServLog will be queried
 * isDeployed: boolean, if true -> periodic update (backend query) will be enabled, if false -> periodic update will be disabled
 */

const LOG_POLL_INTERVAL = 5000;
export function LokiPanel({ host, iocName, isSyslog, isDeployed }) {
  const showWarning = useCustomSnackbar();

  function onError(message) {
    showWarning(message, "warning");
  }

  const hostName = host.csEntryHost.name;
  const [logData, getLogData /* reset*/, , logDataLoading] =
    useLokiSysLog(onError);
  const [procServLog, getProcServLog /* reset*/, , procServLoading] =
    useLokiProcServLog(onError);
  const [timeRange, setTimeRange] = React.useState(720);
  const [timeRangeText, setTimeRangeText] = React.useState("12 hours");
  const [periodChange, setPeriodChange] = useState(false);
  const [alertIds, setAlertIds] = useState([]);

  const handleChange = (event) => {
    setPeriodChange(true);
    setTimeRange(event.target?.value);
    setTimeRangeText(event.currentTarget?.innerText);
  };

  // remove progressBar if intervall has been changed, and data received
  useEffect(() => {
    setPeriodChange(false);
  }, [logData, procServLog]);

  // remove the logline limit exceed warning when user leaves component
  useEffect(() => {
    return () => {
      alertIds.forEach((id) => {
        closeSnackbar(id);
      });
    };
  }, [alertIds]);

  const updateLogs = useCallback(() => {
    if (isSyslog === true) {
      getLogData(hostName, timeRange);
    } else {
      if (isDeployed === true) {
        getProcServLog(hostName, iocName, timeRange);
      }
    }
  }, [
    getLogData,
    getProcServLog,
    hostName,
    iocName,
    isDeployed,
    isSyslog,
    timeRange
  ]);

  useSafePolling(
    updateLogs,
    logDataLoading || procServLoading,
    LOG_POLL_INTERVAL
  );

  // if request period has been changed -> show progressBar
  // if (periodChange) {
  //     return <div style={{ width: "100%" }}><LinearProgress color="primary" /></div>;
  // }

  const dataReady = () => {
    return logData || procServLog;
  };

  let header = (
    <FormControl
      variant="filled"
      className={classes.formControl}
    >
      <InputLabel id="time-range-select">Time range</InputLabel>
      <Select
        variant="standard"
        labelId="time-range-select"
        value={timeRange}
        onChange={handleChange}
      >
        <MenuItem value={720}>12 hours</MenuItem>
        <MenuItem value={10080}>7 days</MenuItem>
      </Select>
    </FormControl>
  );

  return (
    <Root>
      <Container maxWidth="xl">
        {logData || procServLog || !isDeployed ? (
          <Grid
            container
            spacing={1}
            justifyContent="flex-start"
          >
            <Grid
              item
              xs={12}
              md={12}
            >
              <Console
                html={logsToPreprocess(
                  isSyslog,
                  logData,
                  procServLog,
                  isDeployed,
                  showWarning,
                  timeRangeText,
                  alertIds,
                  setAlertIds
                )}
                dataReady={dataReady}
                extraClass={isDeployed ? classes.deployed : classes.undeployed}
                title={isSyslog ? "Syslog" : "ProcServ log"}
                dialogHeader={header}
                loading={periodChange}
              />
            </Grid>
          </Grid>
        ) : (
          <div style={{ width: "100%" }}>
            <LinearProgress color="primary" />
          </div>
        )}
      </Container>
    </Root>
  );
}

function logsToPreprocess(
  isSysLog,
  logData,
  procServLog,
  isDeployed,
  showWarning,
  timeRangeText,
  alertIds,
  setAlertIds
) {
  if (isSysLog === true) {
    return preprocessLog(
      logData,
      true,
      showWarning,
      timeRangeText,
      alertIds,
      setAlertIds
    );
  }

  return preprocessLog(
    procServLog,
    isDeployed,
    showWarning,
    timeRangeText,
    alertIds,
    setAlertIds
  );
}

function formatLogLine(logLine) {
  var Convert = require("ansi-to-html");
  var convert = new Convert();

  return (
    "<span><span class=logdate>" +
    formatDate(logLine.logDate) +
    "</span> " +
    convert.toHtml(logLine.logMessage) +
    "<br/></span>"
  );
}

function preprocessLog(
  logData,
  isDeployed,
  showWarning,
  timeRangeText,
  alertIds,
  setAlertIds
) {
  if (logData.warning && alertIds.length === 0) {
    const warningId = showWarning(logData.warning, "warning");
    setAlertIds((alertIds) => [...alertIds, warningId]);
  }

  if (logData && logData.lines?.length > 0) {
    var logHtml = logData.lines.map((line) => formatLogLine(line));
    return `<html>
                  <head><meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                    <style type="text/css">.logdate { color: #5cb85c; } body.pre{font-family: Monaco, Menlo, Consolas, "Courier New", monospace;}
                    </style>
                  </head>
                  <body>
                    <pre>${logHtml.join("")}</pre>
                  </body>
                </html>`;
  } else {
    if (!(isDeployed === true)) {
      return "<html><body> NOT DEPLOYED YET! </body></html>";
    }
  }

  return `<html><body> - No messages found for ${timeRangeText} period - </body></html>`;
}
